@@ -13,7 +13,7 @@
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "dbpmda.h"
@@ -31,13 +31,26 @@ char		*pmnsfile = PM_NS_DEFAULT;
 char		*cmd_namespace = NULL; /* namespace given from command */
 int             _creds_timeout = 3;     /* Timeout for agents credential PDU */
 
-int		connmode = PDU_NOT;
+int		connmode = NO_CONN;
 int		stmt_type;
 int		eflag;
 int		iflag;
 
 extern int yyparse(void);
 
+/*
+ * called before regular exit() or as atexit() handler
+ */
+static void
+cleanup()
+{
+    if (connmode == CONN_DSO)
+	closedso();
+    else if (connmode == CONN_DAEMON)
+	closepmda();
+    connmode = NO_CONN;
+}
+
 int
 main(int argc, char **argv)
 {
@@ -159,6 +172,10 @@ main(int argc, char **argv)
     setlinebuf(stdout);
     setlinebuf(stderr);
 
+#ifdef HAVE_ATEXIT
+    atexit(cleanup);
+#endif
+
     for ( ; ; ) {
 	initmetriclist();
 	yyparse();
@@ -178,32 +195,32 @@ main(int argc, char **argv)
 
 	    case CLOSE:
 		switch (connmode) {
-		    case PDU_DSO:
+		    case CONN_DSO:
 			closedso();
 			break;
 		    
-		    case PDU_BINARY:
+		    case CONN_DAEMON:
 			closepmda();
 			break;
 		    
-		    case PDU_NOT:
+		    case NO_CONN:
 			yywarn("No PMDA currently opened");
 			break;
 		}
-		connmode = PDU_NOT;
+		connmode = NO_CONN;
 		break;
 
 	    case DESC:
 		switch (connmode) {
-		    case PDU_DSO:
+		    case CONN_DSO:
 			dodso(PDU_DESC_REQ);
 			break;
 		    
-		    case PDU_BINARY:
+		    case CONN_DAEMON:
 			dopmda(PDU_DESC_REQ);
 			break;
 		    
-		    case PDU_NOT:
+		    case NO_CONN:
 			yywarn("No PMDA currently opened");
 			break;
 		}
@@ -211,15 +228,15 @@ main(int argc, char **argv)
 
 	    case FETCH:
 		switch (connmode) {
-		    case PDU_DSO:
+		    case CONN_DSO:
 			dodso(PDU_FETCH);
 			break;
 		    
-		    case PDU_BINARY:
+		    case CONN_DAEMON:
 			dopmda(PDU_FETCH);
 			break;
 		    
-		    case PDU_NOT:
+		    case NO_CONN:
 			yywarn("No PMDA currently opened");
 			break;
 		}
@@ -227,15 +244,15 @@ main(int argc, char **argv)
 
 	    case INSTANCE:
 		switch (connmode) {
-		    case PDU_DSO:
+		    case CONN_DSO:
 			dodso(PDU_INSTANCE_REQ);
 			break;
 		    
-		    case PDU_BINARY:
+		    case CONN_DAEMON:
 			dopmda(PDU_INSTANCE_REQ);
 			break;
 		    
-		    case PDU_NOT:
+		    case NO_CONN:
 			yywarn("No PMDA currently opened");
 			break;
 		}
@@ -243,15 +260,15 @@ main(int argc, char **argv)
 
 	    case STORE:
 		switch (connmode) {
-		    case PDU_DSO:
+		    case CONN_DSO:
 			dodso(PDU_RESULT);
 			break;
 		    
-		    case PDU_BINARY:
+		    case CONN_DAEMON:
 			dopmda(PDU_RESULT);
 			break;
 		    
-		    case PDU_NOT:
+		    case NO_CONN:
 			yywarn("No PMDA currently opened");
 			break;
 		}
@@ -269,7 +286,7 @@ main(int argc, char **argv)
 		break;
 
 	    case QUIT:
-		exit(0);
+		goto done;
 
 	    case STATUS:
 		dostatus();
@@ -277,15 +294,15 @@ main(int argc, char **argv)
 
 	    case INFO:
 		switch (connmode) {
-		case PDU_DSO:
+		case CONN_DSO:
 		    dodso(PDU_TEXT_REQ);
 		    break;
 
-		case PDU_BINARY:
+		case CONN_DAEMON:
 		    dopmda(PDU_TEXT_REQ);
 		    break;
 
-		case PDU_NOT:
+		case NO_CONN:
 		    yywarn("No PMDA currently opened");
 		    break;
 		}
@@ -334,15 +351,15 @@ main(int argc, char **argv)
 
 	    case PMNS_NAME:
 		switch (connmode) {
-		    case PDU_DSO:
+		    case CONN_DSO:
 			dodso(PDU_PMNS_IDS);
 			break;
 		    
-		    case PDU_BINARY:
+		    case CONN_DAEMON:
 			dopmda(PDU_PMNS_IDS);
 			break;
 		    
-		    case PDU_NOT:
+		    case NO_CONN:
 			yywarn("No PMDA currently opened");
 			break;
 		}
@@ -350,15 +367,15 @@ main(int argc, char **argv)
 
 	    case PMNS_PMID:
 		switch (connmode) {
-		    case PDU_DSO:
+		    case CONN_DSO:
 			dodso(PDU_PMNS_NAMES);
 			break;
 		    
-		    case PDU_BINARY:
+		    case CONN_DAEMON:
 			dopmda(PDU_PMNS_NAMES);
 			break;
 		    
-		    case PDU_NOT:
+		    case NO_CONN:
 			yywarn("No PMDA currently opened");
 			break;
 		}
@@ -366,15 +383,15 @@ main(int argc, char **argv)
 
 	    case PMNS_CHILDREN:
 		switch (connmode) {
-		    case PDU_DSO:
+		    case CONN_DSO:
 			dodso(PDU_PMNS_CHILD);
 			break;
 		    
-		    case PDU_BINARY:
+		    case CONN_DAEMON:
 			dopmda(PDU_PMNS_CHILD);
 			break;
 		    
-		    case PDU_NOT:
+		    case NO_CONN:
 			yywarn("No PMDA currently opened");
 			break;
 		}
@@ -382,15 +399,15 @@ main(int argc, char **argv)
 
 	    case PMNS_TRAVERSE:
 		switch (connmode) {
-		    case PDU_DSO:
+		    case CONN_DSO:
 			dodso(PDU_PMNS_TRAVERSE);
 			break;
 		    
-		    case PDU_BINARY:
+		    case CONN_DAEMON:
 			dopmda(PDU_PMNS_TRAVERSE);
 			break;
 		    
-		    case PDU_NOT:
+		    case NO_CONN:
 			yywarn("No PMDA currently opened");
 			break;
 		}
@@ -403,5 +420,8 @@ main(int argc, char **argv)
 	__pmSetInternalState(PM_STATE_APPL);
     }
 
+done:
+    cleanup();
+
     exit(0);
 }
