@@ -10,16 +10,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
-#include <string.h>
-#include <sys/types.h>
 #include <sys/stat.h>
-#include <errno.h>
 #include "./dbpmda.h"
 #include "pmapi.h"
 
@@ -41,7 +34,7 @@ opendso(char *dso, char *init, int domain)
     dispatch.status = -1;
 
     if (stat(dso, &buf) < 0) {
-	fprintf(stderr, "opendso: %s: %s\n", dso, strerror(errno));
+	fprintf(stderr, "opendso: %s: %s\n", dso, osstrerror());
 	return;
     }
    
@@ -145,12 +138,19 @@ opendso(char *dso, char *init, int domain)
 		}
 #endif
 		dsoname = strdup(dso);
-		connmode = PDU_DSO;
+		connmode = CONN_DSO;
 		reset_profile();
 
 		if (myPmdaName != NULL)
 		    free(myPmdaName);
 		myPmdaName = strdup(dso);
+
+		/*
+		 * set here once and used by all subsequent calls into the
+		 * PMDA
+		 */
+		if (dispatch.comm.pmda_interface >= PMDA_INTERFACE_5)
+		    dispatch.version.four.ext->e_context = 0;
 	    }
 	}
     }
@@ -165,12 +165,17 @@ void
 closedso(void)
 {
     if (dsoname != NULL) {
+	if (dispatch.comm.pmda_interface >= PMDA_INTERFACE_5) {
+	    if (dispatch.version.four.ext->e_endCallBack != NULL) {
+		(*(dispatch.version.four.ext->e_endCallBack))(0);
+	    }
+	}
 #ifdef HAVE_DLOPEN
 	dlclose(handle);
 #endif
 	free(dsoname);
 	dsoname = NULL;
-	connmode = PDU_NOT;
+	connmode = NO_CONN;
     }
 }
 
@@ -187,7 +192,7 @@ dodso_desc(pmID pmid, pmDesc *desc)
     if (pmDebug & DBG_TRACE_PDU)
 	fprintf(stderr, "DSO desc()\n");
 #endif
-    if (dispatch.comm.pmda_interface == PMDA_INTERFACE_4)
+    if (dispatch.comm.pmda_interface >= PMDA_INTERFACE_4)
 	sts = dispatch.version.four.desc(pmid, desc, dispatch.version.four.ext);
     else if (dispatch.comm.pmda_interface == PMDA_INTERFACE_3 ||
              dispatch.comm.pmda_interface == PMDA_INTERFACE_2)
@@ -213,7 +218,7 @@ dodso(int pdu)
     pmDesc		desc;
     pmDesc		*desc_list = NULL;	/* initialize to pander to gcc */
     pmResult		*result;
-    __pmInResult		*inresult;
+    __pmInResult	*inresult;
     int			i;
     int			j;
     char		*buffer;
@@ -224,7 +229,7 @@ dodso(int pdu)
     pmID		pmid;
 
     if (timer != 0)
-	gettimeofday(&start, NULL);
+	__pmtimevalNow(&start);
 
     switch (pdu) {
 
@@ -262,7 +267,7 @@ dodso(int pdu)
 		if (pmDebug & DBG_TRACE_PDU)
 		    fprintf(stderr, "DSO profile()\n");
 #endif
-		if (dispatch.comm.pmda_interface == PMDA_INTERFACE_4)
+		if (dispatch.comm.pmda_interface >= PMDA_INTERFACE_4)
 		    sts = dispatch.version.four.profile(profile, dispatch.version.four.ext);
 		else if (dispatch.comm.pmda_interface == PMDA_INTERFACE_3 ||
 		         dispatch.comm.pmda_interface == PMDA_INTERFACE_2)
@@ -283,7 +288,7 @@ dodso(int pdu)
 		if (pmDebug & DBG_TRACE_PDU)
 		    fprintf(stderr, "DSO fetch()\n");
 #endif
-		if (dispatch.comm.pmda_interface == PMDA_INTERFACE_4)
+		if (dispatch.comm.pmda_interface >= PMDA_INTERFACE_4)
 		    sts = dispatch.version.four.fetch(param.numpmid, param.pmidlist, 
 						     &result, dispatch.version.four.ext);
 		else if (dispatch.comm.pmda_interface == PMDA_INTERFACE_3 ||
@@ -307,6 +312,11 @@ dodso(int pdu)
 		    }
                     else
 		        __pmDumpResult(stdout, result);
+		    /*
+		     * DSO PMDA will manage the pmResult skelton, but
+		     * we need to free the pmValueSets and values here
+		     */
+		    __pmFreeResultValues(result);
                 }
 		else {
 		    if (dispatch.comm.pmapi_version == PMAPI_VERSION_1)
@@ -323,7 +333,7 @@ dodso(int pdu)
 		fprintf(stderr, "DSO instance()\n");
 #endif
 
-	    if (dispatch.comm.pmda_interface == PMDA_INTERFACE_4)
+	    if (dispatch.comm.pmda_interface >= PMDA_INTERFACE_4)
 		sts = dispatch.version.four.instance(param.indom, param.number, 
 						    param.name, &inresult,
 						    dispatch.version.four.ext);
@@ -356,7 +366,7 @@ dodso(int pdu)
 
 	    if (profile_changed) {
 		printf("Sending Profile...\n");
-		if (dispatch.comm.pmda_interface == PMDA_INTERFACE_4)
+		if (dispatch.comm.pmda_interface >= PMDA_INTERFACE_4)
 		    sts = dispatch.version.four.profile(profile, dispatch.version.four.ext);
 		else if (dispatch.comm.pmda_interface == PMDA_INTERFACE_3 ||
 			 dispatch.comm.pmda_interface == PMDA_INTERFACE_2)
@@ -375,7 +385,7 @@ dodso(int pdu)
 	    }
 
 	    printf("Getting Result Structure...\n");
-	    if (dispatch.comm.pmda_interface == PMDA_INTERFACE_4)
+	    if (dispatch.comm.pmda_interface >= PMDA_INTERFACE_4)
 		sts = dispatch.version.four.fetch(1, &(desc.pmid), &result,
 						    dispatch.version.four.ext);
 	    else if (dispatch.comm.pmda_interface == PMDA_INTERFACE_3 ||
@@ -410,7 +420,7 @@ dodso(int pdu)
 		return;
 	    }
 
-	    if (dispatch.comm.pmda_interface == PMDA_INTERFACE_4)
+	    if (dispatch.comm.pmda_interface >= PMDA_INTERFACE_4)
 		sts = dispatch.version.four.store(result, dispatch.version.four.ext);
 	    else if (dispatch.comm.pmda_interface == PMDA_INTERFACE_3 ||
 	             dispatch.comm.pmda_interface == PMDA_INTERFACE_2)
@@ -445,7 +455,7 @@ dodso(int pdu)
 		    param.number |= PM_TEXT_HELP;
 		}
 
-		if (dispatch.comm.pmda_interface == PMDA_INTERFACE_4)
+		if (dispatch.comm.pmda_interface >= PMDA_INTERFACE_4)
 		    sts = dispatch.version.four.text(i, param.number, &buffer, dispatch.version.four.ext);
 		else if (dispatch.comm.pmda_interface == PMDA_INTERFACE_3 ||
 			 dispatch.comm.pmda_interface == PMDA_INTERFACE_2)
@@ -571,7 +581,7 @@ dodso(int pdu)
     }
 	   
     if (sts >= 0 && timer != 0) {
-	gettimeofday(&end, NULL);
+	__pmtimevalNow(&end);
 	printf("Timer: %f seconds\n", __pmtimevalSub(&end, &start));
     }
 }
