@@ -22,13 +22,15 @@ extern "C" {
 #endif
 
 /*
- * libpcp_pmda interface versions ... _must_ be bit field values
+ * libpcp_pmda interface versions
  */
 #define PMDA_INTERFACE_1	1	/* initial argument style */
 #define PMDA_INTERFACE_2	2	/* new function arguments */
 #define PMDA_INTERFACE_3	3	/* 3-state return from fetch callback */
 #define PMDA_INTERFACE_4	4	/* dynamic pmns */
-#define PMDA_INTERFACE_LATEST	4
+#define PMDA_INTERFACE_5	5	/* client context in pmda and */
+					/* 4-state return from fetch callback */
+#define PMDA_INTERFACE_LATEST	5
 
 /*
  * Type of I/O connection to PMCD (pmdaUnknown defaults to pmdaPipe)
@@ -66,6 +68,13 @@ typedef struct {
  */
 typedef int (*pmdaFetchCallBack)(pmdaMetric *, unsigned int, pmAtomValue *);
 
+/*
+ * return values for a pmdaFetchCallBack method
+ */
+#define PMDA_FETCH_NOVALUES	0
+#define PMDA_FETCH_STATIC	1
+#define PMDA_FETCH_DYNAMIC	2	/* free avp->vp after __pmStuffValue */
+
 /*
  * Type of function call back used by pmdaMain to clean up a pmResult structure
  * after a fetch.
@@ -84,6 +93,12 @@ typedef int (*pmdaCheckCallBack)(void);
  */
 typedef void (*pmdaDoneCallBack)(void);
 
+/* 
+ * Type of function call back used by pmdaMain when a client context is
+ * closed by PMCD.
+ */
+typedef void (*pmdaEndContextCallBack)(int);
+
 /*
  * libpcp_pmda extension structure.
  *
@@ -122,6 +137,9 @@ typedef struct {
     pmdaFetchCallBack  e_fetchCallBack;  /* callback to assign metric values in fetch */
     pmdaCheckCallBack  e_checkCallBack;  /* callback on receipt of a PDU */
     pmdaDoneCallBack   e_doneCallBack;   /* callback after PDU has been processed */
+    /* added for PMDA_INTERFACE_5 */
+    int		e_context;	/* client context id from pmcd */
+    pmdaEndContextCallBack	e_endCallBack;	/* callback after client context closed */
 } pmdaExt;
 
 /*
@@ -176,8 +194,9 @@ typedef struct {
 	} two;
 
 /*
- * Interface Version 4 (dynamic pmns support)
- * PMDA_INTERFACE_4
+ * Interface Version 4 (dynamic pmns support) and Version 5 (client context
+ * in pmda)
+ * PMDA_INTERFACE_4 and PMDA_INTERFACE_5
  */
 
 	struct {
@@ -188,9 +207,9 @@ typedef struct {
 	    int	    (*instance)(pmInDom, int, char *, __pmInResult **, pmdaExt *);
 	    int	    (*text)(int, int, char **, pmdaExt *);
 	    int	    (*store)(pmResult *, pmdaExt *);
-	    int     (*pmid)(char *, pmID *, pmdaExt *);
+	    int     (*pmid)(const char *, pmID *, pmdaExt *);
 	    int     (*name)(pmID, char ***, pmdaExt *);
-	    int     (*children)(char *, int, char ***, int **, pmdaExt *);
+	    int     (*children)(const char *, int, char ***, int **, pmdaExt *);
 	} four;
 
     } version;
@@ -269,13 +288,20 @@ extern __pmDSO *__pmLookupDSO(int /*domain*/);
  *
  * pmdaSetCheckCallBack
  *      Allows an application specific routine to be called upon receipt of any
- *      PDU. For all PDUs except PDU_PROFILE, a result less than zero indicates an
- *      error. If set to zero (which is also the default), the callback is ignored.
+ *      PDU. For all PDUs except PDU_PROFILE, a result less than zero
+ *      indicates an error. If set to zero (which is also the default),
+ *      the callback is ignored.
  *
  * pmdaSetDoneCallBack
  *      Allows an application specific routine to be called after each PDU is
- *      processed. The result is ignored. If set to zero (which is also the default),
- *      the callback is ignored.
+ *      processed. The result is ignored. If set to zero (which is also
+ *      the default), the callback is ignored.
+ *
+ * pmdaSetEndContextCallBack
+ *      Allows an application specific routine to be called when a
+ *      PMCD context is closed, so any per-context state can be cleaned
+ *      up.  If set to zero (which is also the default), the callback
+ *      is ignored.
  */
 
 extern int pmdaGetOpt(int, char *const *, const char *, pmdaInterface *, int *);
@@ -291,6 +317,7 @@ extern void pmdaSetResultCallBack(pmdaInterface *, pmdaResultCallBack);
 extern void pmdaSetFetchCallBack(pmdaInterface *, pmdaFetchCallBack);
 extern void pmdaSetCheckCallBack(pmdaInterface *, pmdaCheckCallBack);
 extern void pmdaSetDoneCallBack(pmdaInterface *, pmdaDoneCallBack);
+extern void pmdaSetEndContextCallBack(pmdaInterface *, pmdaEndContextCallBack);
 
 /*
  * Callbacks to PMCD which should be adequate for most PMDAs.
@@ -338,9 +365,9 @@ extern int pmdaInstance(pmInDom, int, char *, __pmInResult **, pmdaExt *);
 extern int pmdaDesc(pmID, pmDesc *, pmdaExt *);
 extern int pmdaText(int, int, char **, pmdaExt *);
 extern int pmdaStore(pmResult *, pmdaExt *);
-extern int pmdaPMID(char *, pmID *, pmdaExt *);
+extern int pmdaPMID(const char *, pmID *, pmdaExt *);
 extern int pmdaName(pmID, char ***, pmdaExt *);
-extern int pmdaChildren(char *, int, char ***, int **, pmdaExt *);
+extern int pmdaChildren(const char *, int, char ***, int **, pmdaExt *);
 
 /*
  * PMDA "help" text manipulation
@@ -385,12 +412,19 @@ extern int pmdaTreeSize(__pmnsTree *);
  * 	add entry into the cache, or change state, assigns internal
  * 	instance identifier
  *
+ * pmdaCacheStoreKey
+ * 	add entry into the cache, or change state, caller provides "hint"
+ * 	for internal instance identifier
+ *
  * pmdaCacheLookup
  *	fetch entry based on internal instance identifier
  *
  * pmdaCacheLookupName
  *	fetch entry based on external instance name
  *
+ * pmdaCacheLookupKey
+ *	fetch entry based on key as "hint", like pmdaCacheStoreKey()
+ *
  * pmdaCacheOp
  *	service routines to load, unload, mark as write-thru, purge,
  *	count entries, etc
@@ -399,8 +433,10 @@ extern int pmdaTreeSize(__pmnsTree *);
  *	cull inactive entries
  */
 extern int pmdaCacheStore(pmInDom, int, const char *, void *);
+extern int pmdaCacheStoreKey(pmInDom, int, const char *, int, const void *, void *);
 extern int pmdaCacheLookup(pmInDom, int, char **, void **);
 extern int pmdaCacheLookupName(pmInDom, const char *, int *, void **);
+extern int pmdaCacheLookupKey(pmInDom, const char *, int, const void *, char **, int *, void **);
 extern int pmdaCacheOp(pmInDom, int);
 extern int pmdaCachePurge(pmInDom, time_t);
 
@@ -422,6 +458,7 @@ extern int pmdaCachePurge(pmInDom, time_t);
 #define PMDA_CACHE_REORG		17
 #define PMDA_CACHE_SYNC			18
 #define PMDA_CACHE_DUMP			19
+#define PMDA_CACHE_DUMP_ALL		20
 
 /*
  * Internal libpcp_pmda routines.
@@ -484,16 +521,48 @@ extern void __pmdaCacheDumpAll(FILE *, int);
 extern void __pmdaCacheDump(FILE *, pmInDom, int);
 
 /*
- * Outdated routines
- *
- * pmdaMainLoopFreeResultCallback
- *      Was provided for setting the callback in pmdaMainLoop for cleaning
- *	the pmResult structure.
- *	Do not use this function as this is now supported by 
- *      pmdaSetResultCallBack().
- *
- * extern void pmdaMainLoopFreeResultCallback(void (*)(pmResult *));
+ * Client Context support
  */
+extern int pmdaGetContext(void);
+extern void __pmdaSetContext(int);
+
+/*
+ * Event Record support
+ */
+extern int pmdaEventNewArray(void);
+extern int pmdaEventResetArray(int);
+extern int pmdaEventReleaseArray(int);
+extern int pmdaEventAddRecord(int, struct timeval *, int);
+extern int pmdaEventAddMissedRecord(int, struct timeval *, int);
+extern int pmdaEventAddParam(int, pmID, int, pmAtomValue *);
+extern pmEventArray *pmdaEventGetAddr(int);
+
+/*
+ * Event Queue support
+ */
+extern int pmdaEventNewQueue(const char *, size_t);
+extern int pmdaEventQueueHandle(const char *);
+extern int pmdaEventQueueAppend(int, void *, size_t, struct timeval *);
+extern int pmdaEventQueueClients(int, pmAtomValue *);
+extern int pmdaEventQueueCounter(int, pmAtomValue *);
+extern int pmdaEventQueueBytes(int, pmAtomValue *);
+extern int pmdaEventQueueMemory(int, pmAtomValue *);
+
+typedef int (*pmdaEventDecodeCallBack)(int, void *, size_t, void *);
+extern int pmdaEventQueueRecords(int, pmAtomValue *, int,
+		pmdaEventDecodeCallBack, void *);
+
+extern int pmdaEventNewClient(int);
+extern int pmdaEventEndClient(int);
+extern int pmdaEventClients(pmAtomValue *);
+
+typedef int (*pmdaEventApplyFilterCallBack)(void *, void *, size_t);
+typedef void (*pmdaEventReleaseFilterCallBack)(void *);
+extern int pmdaEventSetFilter(int, int, void *,
+		pmdaEventApplyFilterCallBack, pmdaEventReleaseFilterCallBack);
+extern int pmdaEventSetAccess(int, int, int);
+
+extern char *__pmdaEventPrint(const char *, int, char *, int);
 
 #ifdef __cplusplus
 }
