@@ -10,31 +10,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include <limits.h>
 #include "pmapi.h"
 #include "impl.h"
 
-#if !defined(HAVE_HSTRERROR)
-static char *
-hstrerror(int err)
-{
-    switch (err) {
-	    case 0: return "";
-	    case 1: return "Host not found";
-	    case 2: return "Try again";
-	    case 3: return "Non-recoverable error";
-	    case 4: return "No address";
-	    default: return "Unknown error";
-    }
-}
-#endif
-
 /* Host access control list */
 
 typedef struct {
@@ -50,7 +31,7 @@ typedef struct {
 
 /* Mask of the operations defined by the user of the routines */
 
-static unsigned int	all_ops = 0;	/* mask of all operations specifiable */
+static unsigned int	all_ops;	/* mask of all operations specifiable */
 
 /* This allows the set of valid operations to be specified.
  * Operations must be powers of 2.
@@ -85,7 +66,7 @@ __pmAccAddOp(unsigned int op)
  * checking for incoming connections from localhost.
  */
 
-static int		gotMyHostId = 0;
+static int		gotMyHostId;
 static struct in_addr	myHostId;
 static char		myHostName[MAXHOSTNAMELEN+1];
 
@@ -99,9 +80,8 @@ GetMyHostId(void)
     myHostName[MAXHOSTNAMELEN-1] = '\0';
 
     if ((hep = gethostbyname(myHostName)) == NULL) {
-	__pmNotifyErr(LOG_ERR,
-		     "gethostbyname(%s), %s\n",
-		     myHostName, hstrerror(h_errno));
+	__pmNotifyErr(LOG_ERR, "gethostbyname(%s), %s\n",
+		     myHostName, hoststrerror());
 	return -1;
     }
     myHostId.s_addr = ((struct in_addr *)hep->h_addr_list[0])->s_addr;
@@ -111,13 +91,13 @@ GetMyHostId(void)
 
 /* This is the host access list */
 
-static HostInfo	*hostList = NULL;
-static int	nHosts = 0;
-static int	szHostList = 0;
+static HostInfo	*hostList;
+static int	nHosts;
+static int	szHostList;
 
 /* Used for saving the current state of the host access list */
 
-static int	saved = 0;
+static int	saved;
 static HostInfo	*oldHostList;
 static int	oldNHosts;
 static int	oldSzHostList;
@@ -212,7 +192,7 @@ __pmAccFreeSavedHosts(void)
  *	id.  Zero means unspecified, which will allow unlimited connections or
  *	a subsequent __pmAccAddHost call with the same host to override maxCons.
  *
- * Returns a negated errno code on failure.
+ * Returns a negated system error code on failure.
  */
 
 int
@@ -298,10 +278,9 @@ __pmAccAddHost(const char *name, unsigned int specOps, unsigned int denyOps, int
 	else
 	    realName = name;
 	if ((hep = gethostbyname(realName)) == NULL) {
-	    __pmNotifyErr(LOG_ERR,
-			 "gethostbyname(%s), %s\n",
-			 realName, hstrerror(h_errno));
-	    return -EHOSTUNREACH;	/* h_errno isn't suitable to return */
+	    __pmNotifyErr(LOG_ERR, "gethostbyname(%s), %s\n",
+			 realName, hoststrerror());
+	    return -EHOSTUNREACH;	/* host error unsuitable to return */
 	}
 	hostId.s_addr = ((struct in_addr *)hep->h_addr_list[0])->s_addr;
 	hostMask.s_addr = 0xffffffff;
