@@ -11,10 +11,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
  * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
- *
  * Debug Flags
  *	DERIVE - high-level diagnostics
  *	DERIVE & APPL0 - configuration and static syntax analysis
@@ -37,6 +33,7 @@
  * 	metric persists for the life of the application.
  */
 
+#include <inttypes.h>
 #include "derive.h"
 
 static int	need_init = 1;
@@ -50,7 +47,7 @@ static int	lexpeek = 0;
 static char	*string;
 static char	*errmsg;
 
-static char *type_dbg[] = { "ERROR", "EOF", "UNDEF", "NUMBER", "NAME", "PLUS", "MINUS", "STAR", "SLASH", "LPAREN", "RPAREN", "AVG", "COUNT", "DELTA", "MAX", "MIN", "SUM" };
+static char *type_dbg[] = { "ERROR", "EOF", "UNDEF", "NUMBER", "NAME", "PLUS", "MINUS", "STAR", "SLASH", "LPAREN", "RPAREN", "AVG", "COUNT", "DELTA", "MAX", "MIN", "SUM", "ANON" };
 static char type_c[] = { '\0', '\0', '\0', '\0', '\0', '+', '-', '*', '/', '(', ')', '\0' };
 
 /* function table for lexer */
@@ -64,6 +61,7 @@ static struct {
     { L_MAX,	"max" },
     { L_MIN,	"min" },
     { L_SUM,	"sum" },
+    { L_ANON,	"anon" },
     { L_UNDEF,	NULL }
 };
 
@@ -78,6 +76,43 @@ static struct {
 
 static char *state_dbg[] = { "INIT", "LEAF", "LEAF_PAREN", "BINOP", "FUNC_OP", "FUNC_END" };
 
+/* Register an anonymous metric */
+int
+__pmRegisterAnon(char *name, int type)
+{
+    char	*msg;
+    char	buf[21];	/* anon(PM_TYPE_XXXXXX) */
+
+    switch (type) {
+	case PM_TYPE_32:
+	    snprintf(buf, sizeof(buf), "anon(PM_TYPE_32)");
+	    break;
+	case PM_TYPE_U32:
+	    snprintf(buf, sizeof(buf), "anon(PM_TYPE_U32)");
+	    break;
+	case PM_TYPE_64:
+	    snprintf(buf, sizeof(buf), "anon(PM_TYPE_64)");
+	    break;
+	case PM_TYPE_U64:
+	    snprintf(buf, sizeof(buf), "anon(PM_TYPE_U64)");
+	    break;
+	case PM_TYPE_FLOAT:
+	    snprintf(buf, sizeof(buf), "anon(PM_TYPE_FLOAT)");
+	    break;
+	case PM_TYPE_DOUBLE:
+	    snprintf(buf, sizeof(buf), "anon(PM_TYPE_DOUBLE)");
+	    break;
+	default:
+	    return PM_ERR_TYPE;
+    }
+    if ((msg = pmRegisterDerived(name, buf)) != NULL) {
+	pmprintf("__pmRegisterAnon(%s, %d): Error: %s\n", name, type, pmDerivedErrStr());
+	pmflush();
+	return PM_ERR_GENERIC;
+    }
+    return 0;
+}
+
 static void
 init(void)
 {
@@ -376,6 +411,7 @@ void report_sem_error(char *name, node_t *np)
 	case L_MAX:
 	case L_MIN:
 	case L_SUM:
+	case L_ANON:
 	    pmprintf("%s(%s)", type_dbg[np->type+2], np->left->value);
 	    break;
 	default:
@@ -546,8 +582,9 @@ map_desc(int n, node_t *np)
      * PM_SEM_DISCRETE in which case the result is also PM_SEM_DISCRETE
      *
      * type promotion (similar to ANSI C)
-     * PM_TYPE_STRING, PM_TYPE_AGGREGATE and PM_TYPE_AGGREGATE_STATIC are
-     * illegal operands except for renaming (no operator involved)
+     * PM_TYPE_STRING, PM_TYPE_AGGREGATE, PM_TYPE_AGGREGATE_STATIC
+     * and PM_TYPE_EVENT are illegal operands except for renaming
+     * (where no operator is involved)
      * for all operands, division => PM_TYPE_DOUBLE
      * else PM_TYPE_DOUBLE & any type => PM_TYPE_DOUBLE
      * else PM_TYPE_FLOAT & any type => PM_TYPE_FLOAT
@@ -727,7 +764,7 @@ check_expr(int n, node_t *np)
 	/*
 	 * special cases for functions ...
 	 * delta	expect numeric operand, result is instantaneous
-	 * aggr funcs	expect numeric operand, result is instantaneous
+	 * aggr funcs	most expect numeric operand, result is instantaneous
 	 *		and singular
 	 */
 	if (np->type == L_AVG || np->type == L_COUNT || np->type == L_DELTA
@@ -764,6 +801,10 @@ check_expr(int n, node_t *np)
 		np->desc.type = PM_TYPE_FLOAT;
 	    }
 	}
+	else if (np->type == L_ANON) {
+	    /* do nothing, pmDesc inherited "as is" from left node */
+	    ;
+	}
     }
     else {
 	/* build pmDesc from pmDesc of both operands */
@@ -787,11 +828,11 @@ dump_value(int type, pmAtomValue *avp)
 	    break;
 
 	case PM_TYPE_64:
-	    fprintf(stderr, "%lli", (long long)avp->ll);
+	    fprintf(stderr, "%" PRId64, avp->ll);
 	    break;
 
 	case PM_TYPE_U64:
-	    fprintf(stderr, "%llu", (unsigned long long)avp->ull);
+	    fprintf(stderr, "%" PRIu64, avp->ull);
 	    break;
 
 	case PM_TYPE_FLOAT:
@@ -807,6 +848,8 @@ dump_value(int type, pmAtomValue *avp)
 	    break;
 
 	case PM_TYPE_AGGREGATE:
+	case PM_TYPE_AGGREGATE_STATIC:
+	case PM_TYPE_EVENT:
 	case PM_TYPE_UNKNOWN:
 	    fprintf(stderr, "[blob]");
 	    break;
@@ -823,14 +866,15 @@ dump_value(int type, pmAtomValue *avp)
 void
 __dmdumpexpr(node_t *np, int level)
 {
-    if (level == 0) fprintf(stderr, "Derived metric expr dump from %p...\n", np);
+    if (level == 0) fprintf(stderr, "Derived metric expr dump from " PRINTF_P_PFX "%p...\n", np);
     if (np == NULL) return;
-    fprintf(stderr, "expr node %p type=%s left=%p right=%p save_last=%d", np, type_dbg[np->type+2], np->left, np->right, np->save_last);
+    fprintf(stderr, "expr node " PRINTF_P_PFX "%p type=%s left=" PRINTF_P_PFX "%p right=" PRINTF_P_PFX "%p save_last=%d", np, type_dbg[np->type+2], np->left, np->right, np->save_last);
     if (np->type == L_NAME || np->type == L_NUMBER)
 	fprintf(stderr, " [%s] master=%d", np->value, np->info == NULL ? 1 : 0);
     fputc('\n', stderr);
     if (np->info) {
-	fprintf(stderr, "    PMID: %s (%s from pmDesc) numval: %d", pmIDStr(np->info->pmid), pmIDStr(np->desc.pmid), np->info->numval);
+	fprintf(stderr, "    PMID: %s", pmIDStr(np->info->pmid));
+	fprintf(stderr, " (%s from pmDesc) numval: %d", pmIDStr(np->desc.pmid), np->info->numval);
 	if (np->info->div_scale != 1)
 	    fprintf(stderr, " div_scale: %d", np->info->div_scale);
 	if (np->info->mul_scale != 1)
@@ -956,7 +1000,8 @@ parse(int level)
 		    state = P_LEAF_PAREN;
 		}
 		else if (type == L_AVG || type == L_COUNT || type == L_DELTA
-		         || type == L_MAX || type == L_MIN || type == L_SUM) {
+		         || type == L_MAX || type == L_MIN || type == L_SUM 
+			 || type == L_ANON) {
 		    expr = curr = newnode(type);
 		    state = P_FUNC_OP;
 		}
@@ -973,6 +1018,7 @@ parse(int level)
 			curr->type == L_AVG || curr->type == L_COUNT ||
 			curr->type == L_DELTA || curr->type == L_MAX ||
 			curr->type == L_MIN || curr->type == L_SUM ||
+			curr->type == L_ANON ||
 		        type == L_PLUS || type == L_MINUS) {
 			/*
 			 * first operator or equal or lower precedence
@@ -1024,7 +1070,8 @@ parse(int level)
 		    state = P_LEAF_PAREN;
 		}
 		else if (type == L_AVG || type == L_COUNT || type == L_DELTA
-		         || type == L_MAX || type == L_MIN || type == L_SUM) {
+		         || type == L_MAX || type == L_MIN || type == L_SUM
+			 || type == L_ANON) {
 		    np = newnode(type);
 		    curr->right = np;
 		    curr = np;
@@ -1044,6 +1091,38 @@ parse(int level)
 			/*NOTREACHED*/
 		    }
 		    np->save_last = 1;
+		    if (curr->type == L_ANON) {
+			/*
+			 * anon(PM_TYPE_...) is a special case ... the
+			 * argument defines the metric type, the remainder
+			 * of the metadata is fixed and there are never any
+			 * values available. So we build the pmDesc and then
+			 * clobber the "left" node and prevent any attempt to
+			 * contact a PMDA for metadata or values
+			 */
+			if (strcmp(np->value, "PM_TYPE_32") == 0)
+			    np->desc.type = PM_TYPE_32;
+			else if (strcmp(np->value, "PM_TYPE_U32") == 0)
+			    np->desc.type = PM_TYPE_U32;
+			else if (strcmp(np->value, "PM_TYPE_64") == 0)
+			    np->desc.type = PM_TYPE_64;
+			else if (strcmp(np->value, "PM_TYPE_U64") == 0)
+			    np->desc.type = PM_TYPE_U64;
+			else if (strcmp(np->value, "PM_TYPE_FLOAT") == 0)
+			    np->desc.type = PM_TYPE_FLOAT;
+			else if (strcmp(np->value, "PM_TYPE_DOUBLE") == 0)
+			    np->desc.type = PM_TYPE_DOUBLE;
+			else {
+			    fprintf(stderr, "Error: type=%s not allowed for anon()\n", np->value);
+			    free_expr(np);
+			    return NULL;
+			}
+			np->desc.pmid = PM_ID_NULL;
+			np->desc.indom = PM_INDOM_NULL;
+			np->desc.sem = PM_SEM_DISCRETE;
+			memset((void *)&np->desc.units, 0, sizeof(np->desc.units));
+			np->type = L_NUMBER;
+		    }
 		    curr->left = np;
 		    curr = expr;
 		    state = P_FUNC_END;
@@ -1159,7 +1238,7 @@ pmLoadDerivedConfig(char *fname)
 #endif
 
     if ((fp = fopen(fname, "r")) == NULL) {
-	return -errno;
+	return -oserror();
     }
     buflen = 128;
     if ((buf = (char *)malloc(buflen)) == NULL) {
@@ -1262,6 +1341,7 @@ next_line:
 	else
 	    *p++ = c;
     }
+    free(buf);
     return sts;
 }
 
@@ -1418,7 +1498,7 @@ __dmgetname(pmID pmid, char ** name)
 	if (pmid == registered.mlist[i].pmid) {
 	    *name = strdup(registered.mlist[i].name);
 	    if (*name == NULL)
-		return -errno;
+		return -oserror();
 	    else
 		return 0;
 	}
@@ -1437,7 +1517,7 @@ __dmopencontext(__pmContext *ctxp)
 
 #ifdef PCP_DEBUG
     if ((pmDebug & DBG_TRACE_DERIVE) && (pmDebug & DBG_TRACE_APPL1)) {
-	fprintf(stderr, "__dmopencontext() called\n");
+	fprintf(stderr, "__dmopencontext(->ctx %d) called\n", __pmPtrToHandle(ctxp));
     }
 #endif
     if (registered.nmetric == 0) {
@@ -1492,7 +1572,7 @@ __dmclosecontext(__pmContext *ctxp)
 
 #ifdef PCP_DEBUG
     if (pmDebug & DBG_TRACE_DERIVE) {
-	fprintf(stderr, "__dmclosecontext() called dm->%p %d metrics\n", cp, cp == NULL ? -1 : cp->nmetric);
+	fprintf(stderr, "__dmclosecontext(->ctx %d) called dm->" PRINTF_P_PFX "%p %d metrics\n", __pmPtrToHandle(ctxp), cp, cp == NULL ? -1 : cp->nmetric);
     }
 #endif
     if (cp == NULL) return;
