@@ -10,17 +10,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <ctype.h>
-#include <string.h>
-#include <errno.h>
 #include <assert.h>
 #include "pmapi.h"
 #include "impl.h"
@@ -92,5 +84,6 @@ typedef struct {
 #define L_MAX		12
 #define L_MIN		13
 #define L_SUM		14
+#define L_ANON		15
 
 void __dmdumpexpr(node_t *, int);
