@@ -10,21 +10,14 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
-
 #include <stdio.h>
 #include <sys/time.h>
 #include "pmapi.h"
 #include "impl.h"
 #include "pmda.h"
 
-extern int	errno;
-
 int
 __pmFetchLocal(int numpmid, pmID pmidlist[], pmResult **result)
 {
@@ -63,7 +56,7 @@ __pmFetchLocal(int numpmid, pmID pmidlist[], pmResult **result)
      */
     need = (int)sizeof(pmResult) + (numpmid - 1) * (int)sizeof(pmValueSet *);
     if ((ans = (pmResult *)malloc(need)) == NULL)
-	return -errno;
+	return -oserror();
 
     /*
      * Check if we have enough space to accomodate "best" case scenario -
@@ -74,12 +67,12 @@ __pmFetchLocal(int numpmid, pmID pmidlist[], pmResult **result)
 	if ((splitlist = (pmID *)realloc (splitlist,
 					  sizeof (pmID)*splitmax)) == NULL) {
 	    splitmax = 0;
-	    return -errno;
+	    return -oserror();
 	}
     }
 
     ans->numpmid = numpmid;
-    gettimeofday(&ans->timestamp, NULL);
+    __pmtimevalNow(&ans->timestamp);
     for (j = 0; j < numpmid; j++)
 	ans->vset[j] = NULL;
 
@@ -109,7 +102,9 @@ __pmFetchLocal(int numpmid, pmID pmidlist[], pmResult **result)
 			    "__pmFetchLocal: calling ???_profile(domain: %d), "
 			    "context: %d\n", dp->domain, ctx);
 #endif
-		if (dp->dispatch.comm.pmda_interface == PMDA_INTERFACE_4)
+		if (dp->dispatch.comm.pmda_interface >= PMDA_INTERFACE_5)
+		    dp->dispatch.version.four.ext->e_context = ctx;
+		if (dp->dispatch.comm.pmda_interface >= PMDA_INTERFACE_4)
 		    sts = dp->dispatch.version.four.profile(ctxp->c_instprof,
 							   dp->dispatch.version.four.ext);
 		else if (dp->dispatch.comm.pmda_interface == PMDA_INTERFACE_3 ||
@@ -132,7 +127,9 @@ __pmFetchLocal(int numpmid, pmID pmidlist[], pmResult **result)
 	}
 
 	if (sts >= 0) {
-	    if (dp->dispatch.comm.pmda_interface == PMDA_INTERFACE_4)
+	    if (dp->dispatch.comm.pmda_interface >= PMDA_INTERFACE_5)
+		dp->dispatch.version.four.ext->e_context = ctx;
+	    if (dp->dispatch.comm.pmda_interface >= PMDA_INTERFACE_4)
 		sts = dp->dispatch.version.four.fetch(cnt, splitlist, &tmp_ans,
 						     dp->dispatch.version.four.ext);
 	    else if (dp->dispatch.comm.pmda_interface == PMDA_INTERFACE_3 ||
@@ -153,7 +150,7 @@ __pmFetchLocal(int numpmid, pmID pmidlist[], pmResult **result)
 		if (sts < 0) {
 		    ans->vset[k] = (pmValueSet *)malloc(sizeof(pmValueSet));
 		    if (ans->vset[k] == NULL)
-			return -errno;
+			return -oserror();
 		    ans->vset[k]->numval = sts;
 		    ans->vset[k]->pmid = pmidlist[k];
 		}
