@@ -10,10 +10,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include <limits.h>
@@ -396,7 +392,7 @@ update_bounds(__pmContext *ctxp, double t_req, pmResult *logrp, int do_mark, int
 		    }
 #ifdef PCP_DEBUG
 		    if ((pmDebug & DBG_TRACE_INTERP) && changed) {
-			fprintf(stderr, "update%s pmid %s inst %d prior: t==%.3f",
+			fprintf(stderr, "update%s pmid %s inst %d prior: t=%.3f",
 			    changed & 2 ? "+search" : "",
 			    pmIDStr(logrp->vset[k]->pmid), icp->inst, icp->t_prior);
 			if (icp->m_prior)
@@ -472,6 +468,20 @@ do_roll(__pmContext *ctxp, double t_req)
     }
 }
 
+#define pmXTBdeltaToTimeval(d, m, t) { \
+    (t)->tv_sec = 0; \
+    (t)->tv_usec = (long)0; \
+    switch(PM_XTB_GET(m)) { \
+    case PM_TIME_NSEC: (t)->tv_usec = (long)((d) / 1000); break; \
+    case PM_TIME_USEC: (t)->tv_usec = (long)(d); break; \
+    case PM_TIME_MSEC: (t)->tv_sec = (d) / 1000; (t)->tv_usec = (long)(1000 * ((d) % 1000)); break; \
+    case PM_TIME_SEC: (t)->tv_sec = (d); break; \
+    case PM_TIME_MIN: (t)->tv_sec = (d) * 60; break; \
+    case PM_TIME_HOUR: (t)->tv_sec = (d) * 360; break; \
+    default: (t)->tv_sec = (d) / 1000; (t)->tv_usec = (long)(1000 * ((d) % 1000)); break; \
+    } \
+}
+
 int
 __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **result)
 {
@@ -484,7 +494,7 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
     pmResult	*logrp;
     __pmHashCtl	*hcp = &ctxp->c_archctl->ac_pmid_hc;
     __pmHashNode	*hp;
-    pmidcntl_t	*pcp;
+    pmidcntl_t	*pcp = NULL;	/* initialize to pander to gcc */
     instcntl_t	*icp;
     int		back = 0;
     int		forw = 0;
@@ -545,7 +555,7 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
     }
 
     if ((rp = (pmResult *) malloc(sizeof(pmResult) + (numpmid - 1) * sizeof(pmValueSet *))) == NULL)
-	return -errno;
+	return -oserror();
 
     rp->timestamp.tv_sec = ctxp->c_origin.tv_sec;
     rp->timestamp.tv_usec = ctxp->c_origin.tv_usec;
@@ -576,6 +586,7 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
 	    /* first time we've been asked for this one in this context */
 	    if ((pcp = (pmidcntl_t *)malloc(sizeof(pmidcntl_t))) == NULL) {
 		__pmNoMem("__pmLogFetchInterp.pmidcntl_t", sizeof(pmidcntl_t), PM_FATAL_ERR);
+		/*NOTREACHED*/
 	    }
 	    pcp->valfmt = -1;
 	    pcp->first = NULL;
@@ -637,6 +648,9 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
 	if (pcp->desc.type == -1) {
 	    pcp->numval = PM_ERR_PMID_LOG;
 	}
+	else if (pcp->desc.type == PM_TYPE_EVENT) {
+	    pcp->numval = PM_ERR_TYPE;
+	}
 	else if (pcp->desc.indom != PM_INDOM_NULL) {
 	    /* use the profile to filter the instances to be returned */
 	    for (icp = pcp->first; icp != NULL; icp = icp->next) {
@@ -972,7 +986,7 @@ retry_forw:
 			pcp->desc.type != PM_TYPE_U64 &&
 			pcp->desc.type != PM_TYPE_FLOAT &&
 			pcp->desc.type != PM_TYPE_DOUBLE)
-			    pcp->numval = PM_ERR_VALUE;
+			    pcp->numval = PM_ERR_TYPE;
 		}
 	    }
 	}
@@ -1137,7 +1151,7 @@ retry_forw:
 
 		    need = PM_VAL_HDR_SIZE + sizeof(float);
 		    if ((vp = (pmValueBlock *)malloc(need)) == NULL) {
-			sts = -errno;
+			sts = -oserror();
 			goto bad_alloc;
 		    }
 		    vp->vlen = need;
@@ -1194,7 +1208,7 @@ retry_forw:
 
 		    need = PM_VAL_HDR_SIZE + sizeof(__int64_t);
 		    if ((vp = (pmValueBlock *)__pmPoolAlloc(need)) == NULL) {
-			sts = -errno;
+			sts = -oserror();
 			goto bad_alloc;
 		    }
 		    vp->vlen = need;
@@ -1333,7 +1347,7 @@ retry_forw:
 
 		    need = PM_VAL_HDR_SIZE + sizeof(double);
 		    if ((vp = (pmValueBlock *)__pmPoolAlloc(need)) == NULL) {
-			sts = -errno;
+			sts = -oserror();
 			goto bad_alloc;
 		    }
 		    vp->vlen = need;
@@ -1396,7 +1410,7 @@ retry_forw:
 		    else
 			vp = (pmValueBlock *)malloc(need);
 		    if (vp == NULL) {
-			sts = -errno;
+			sts = -oserror();
 			goto bad_alloc;
 		    }
 		    rp->vset[j]->valfmt = PM_VAL_DPTR;
