@@ -10,10 +10,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include "pmapi.h"
@@ -27,6 +23,38 @@
 #include <netinet/tcp.h>
 #endif
 
+/*
+ * Return timeout (in seconds) to be used when pmlc is communicating
+ * with pmlogger ... used externally from pmlc and internally from
+ * __pmConnectLogger() and __pmControlLogger()
+ */
+int
+__pmLoggerTimeout(void)
+{
+    static int		timeout = TIMEOUT_NEVER;
+    static int		done_default = 0;
+
+    if (!done_default) {
+	char	*timeout_str;
+	char	*end_ptr;
+	if ((timeout_str = getenv("PMLOGGER_REQUEST_TIMEOUT")) != NULL) {
+	    /*
+	     * Only a positive integer (the unit is seconds) is OK
+	     */
+	    timeout = strtol(timeout_str, &end_ptr, 10);
+	    if (*end_ptr != '\0' || timeout < 0) {
+		__pmNotifyErr(LOG_WARNING,
+			      "ignored bad PMLOGGER_REQUEST_TIMEOUT = '%s'\n",
+			      timeout_str);
+		timeout = TIMEOUT_NEVER;
+	    }
+	}
+	done_default = 1;
+    }
+
+    return timeout;
+}
+
 /*
  * expect one of pid or port to be 0 ... if port is 0, use
  * hostname+pid to find port, assuming pmcd is running there
@@ -95,7 +123,7 @@ __pmConnectLogger(const char *hostname, int *pid, int *port)
 #ifdef PCP_DEBUG
 	if (pmDebug & DBG_TRACE_CONTEXT)
 	    fprintf(stderr, "__pmConnectLogger: gethostbyname: %s\n",
-		    hstrerror(h_errno));
+		    hoststrerror());
 #endif
 	return -ECONNREFUSED;
     }
@@ -111,7 +139,7 @@ __pmConnectLogger(const char *hostname, int *pid, int *port)
 
     sts = connect(fd, (struct sockaddr*) &myAddr, sizeof(myAddr));
     if (sts < 0) {
-	sts = -errno;
+	sts = -neterror();
 	__pmCloseSocket(fd);
 #ifdef PCP_DEBUG
 	if (pmDebug & DBG_TRACE_CONTEXT)
@@ -121,10 +149,10 @@ __pmConnectLogger(const char *hostname, int *pid, int *port)
     }
 
     /* Expect an error PDU back: ACK/NACK for connection */
-    sts = __pmGetPDU(fd, PDU_BINARY, TIMEOUT_NEVER, &pb);
+    sts = __pmGetPDU(fd, ANY_SIZE, __pmLoggerTimeout(), &pb);
     if (sts == PDU_ERROR) {
 	__pmOverrideLastFd(PDU_OVERRIDE2);	/* don't dink with the value */
-	__pmDecodeError(pb, PDU_BINARY, &sts);
+	__pmDecodeError(pb, &sts);
 	if (sts == 0)
 	    sts = LOG_PDU_VERSION1;
 	else if (sts == PM_ERR_V1(PM_ERR_CONNLIMIT) ||
@@ -146,10 +174,22 @@ __pmConnectLogger(const char *hostname, int *pid, int *port)
 	}
 	*pid = php->from;
     }
+    else if (sts < 0) {
+#ifdef PCP_DEBUG
+	if (pmDebug & DBG_TRACE_CONTEXT) {
+	    if (sts == PM_ERR_TIMEOUT)
+		fprintf(stderr, "__pmConnectLogger: timeout (after %d secs)\n", __pmLoggerTimeout());
+	    else
+		fprintf(stderr, "__pmConnectLogger: Error: %s\n", pmErrStr(sts));
+	}
+#endif
+	;	/* fall through */
+    }
     else {
+	/* wrong PDU type! */
 #ifdef PCP_DEBUG
 	if (pmDebug & DBG_TRACE_CONTEXT)
-	    fprintf(stderr, "__pmConnectLogger: ACK PDU type=%d?\n", sts);
+	    fprintf(stderr, "__pmConnectLogger: ACK botch PDU type=%d not PDU_ERROR?\n", sts);
 #endif
 	sts = PM_ERR_IPC;
     }
@@ -169,7 +209,7 @@ __pmConnectLogger(const char *hostname, int *pid, int *port)
 	    handshake[0].c_vala = LOG_PDU_VERSION;
 	    handshake[0].c_valb = 0;
 	    handshake[0].c_valc = 0;
-	    sts = __pmSendCreds(fd, PDU_BINARY, 1, handshake);
+	    sts = __pmSendCreds(fd, (int)getpid(), 1, handshake);
 	}
 	if (sts >= 0) {
 #ifdef PCP_DEBUG
