@@ -10,10 +10,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include "pmapi.h"
@@ -33,11 +29,11 @@ StrTimeval(__pmTimeval *tp)
     }
     else {
 	static char		sbuf[13];
-	static struct tm	*tmp;
+	struct tm		tmp;
 	time_t 			t = tp->tv_sec;
-	tmp = localtime(&t);
+	pmLocaltime(&t, &tmp);
 	sprintf(sbuf, "%02d:%02d:%02d.%03d",	/* safe */
-	    tmp->tm_hour, tmp->tm_min, tmp->tm_sec, tp->tv_usec/1000);
+	    tmp.tm_hour, tmp.tm_min, tmp.tm_sec, tp->tv_usec/1000);
 	return sbuf;
     }
 }
@@ -70,7 +66,7 @@ __pmHashAdd(unsigned int key, void *data, __pmHashCtl *hcp)
 	hcp->hsize = 1;	/* arbitrary number */
 	if ((hcp->hash = (__pmHashNode **)calloc(hcp->hsize, sizeof(__pmHashNode *))) == NULL) {
 	    hcp->hsize = 0;
-	    return -errno;
+	    return -oserror();
 	}
     }
     else if (hcp->nodes / 4 > hcp->hsize) {
@@ -85,7 +81,7 @@ __pmHashAdd(unsigned int key, void *data, __pmHashCtl *hcp)
 	if ((hcp->hash = (__pmHashNode **)calloc(hcp->hsize, sizeof(__pmHashNode *))) == NULL) {
 	    hcp->hsize = oldsize;
 	    hcp->hash = old;
-	    return -errno;
+	    return -oserror();
 	}
 	/*
 	 * re-link chains
@@ -103,7 +99,7 @@ __pmHashAdd(unsigned int key, void *data, __pmHashCtl *hcp)
     }
 
     if ((hp = (__pmHashNode *)malloc(sizeof(__pmHashNode))) == NULL)
-	return -errno;
+	return -oserror();
 
     k = key % hcp->hsize;
     hp->key = key;
@@ -147,7 +143,7 @@ addindom(__pmLogCtl *lcp, pmInDom indom, const __pmTimeval *tp, int numinst,
     int		sts;
 
     if ((idp = (__pmLogInDom *)malloc(sizeof(__pmLogInDom))) == NULL)
-	return -errno;
+	return -oserror();
     idp->stamp = *tp;		/* struct assignment */
     idp->numinst = numinst;
     idp->instlist = instlist;
@@ -221,7 +217,7 @@ __pmLogLoadMeta(__pmLogCtl *lcp)
 #endif
 	    if (ferror(f)) {
 		clearerr(f);
-		sts = -errno;
+		sts = -oserror();
 	    }
 	    else
 		sts = PM_ERR_LOGREC;
@@ -237,7 +233,7 @@ __pmLogLoadMeta(__pmLogCtl *lcp)
 	if (h.type == TYPE_DESC) {
             numpmid++;
 	    if ((dp = (pmDesc *)malloc(sizeof(pmDesc))) == NULL) {
-		sts = -errno;
+		sts = -oserror();
 		goto end;
 	    }
 	    if ((n = (int)fread(dp, 1, sizeof(pmDesc), f)) != sizeof(pmDesc)) {
@@ -249,7 +245,7 @@ __pmLogLoadMeta(__pmLogCtl *lcp)
 #endif
 		if (ferror(f)) {
 		    clearerr(f);
-		    sts = -errno;
+		    sts = -oserror();
 		}
 		else
 		    sts = PM_ERR_LOGREC;
@@ -284,7 +280,7 @@ __pmLogLoadMeta(__pmLogCtl *lcp)
 #endif
 		    if (ferror(f)) {
 			clearerr(f);
-			sts = -errno;
+			sts = -oserror();
 		    }
 		    else
 			sts = PM_ERR_LOGREC;
@@ -306,7 +302,7 @@ __pmLogLoadMeta(__pmLogCtl *lcp)
 #endif
 			if (ferror(f)) {
 			    clearerr(f);
-			    sts = -errno;
+			    sts = -oserror();
 			}
 			else
 			    sts = PM_ERR_LOGREC;
@@ -326,7 +322,7 @@ __pmLogLoadMeta(__pmLogCtl *lcp)
 #endif
 			if (ferror(f)) {
 			    clearerr(f);
-			    sts = -errno;
+			    sts = -oserror();
 			}
 			else
 			    sts = PM_ERR_LOGREC;
@@ -341,7 +337,16 @@ __pmLogLoadMeta(__pmLogCtl *lcp)
 #endif
 
                     if ((sts = __pmAddPMNSNode(lcp->l_pmns, dp->pmid, name)) < 0) {
-			goto end;
+			/*
+			 * If we see a duplicate PMID, its a recoverable error.
+			 * We wont be able to see all of the data in the log, but
+			 * its better to provide access to some rather than none,
+			 * esp. when only one or two metric IDs may be corrupted
+			 * in this way (which we may not be interested in anyway).
+			 */
+			if (sts != PM_ERR_PMID)
+			    goto end;
+			sts = 0;
 		    } 
 		}/*for*/
             }/*version2*/
@@ -360,7 +365,7 @@ __pmLogLoadMeta(__pmLogCtl *lcp)
 	    int			allinbuf;
 
 	    if ((tbuf = (int *)malloc(rlen)) == NULL) {
-		sts = -errno;
+		sts = -oserror();
 		goto end;
 	    }
 	    if ((n = (int)fread(tbuf, 1, rlen, f)) != rlen) {
@@ -372,7 +377,7 @@ __pmLogLoadMeta(__pmLogCtl *lcp)
 #endif
 		if (ferror(f)) {
 		    clearerr(f);
-		    sts = -errno;
+		    sts = -oserror();
 		}
 		else
 		    sts = PM_ERR_LOGREC;
@@ -398,7 +403,7 @@ __pmLogLoadMeta(__pmLogCtl *lcp)
 		/* need to allocate to hold the pointers */
 		namelist = (char **)malloc(numinst*sizeof(char*));
 		if (namelist == NULL) {
-		    sts = -errno;
+		    sts = -oserror();
 		    goto end;
 		}
 #endif
@@ -430,7 +435,7 @@ __pmLogLoadMeta(__pmLogCtl *lcp)
 #endif
 	    if (ferror(f)) {
 		clearerr(f);
-		sts = -errno;
+		sts = -oserror();
 	    }
 	    else
 		sts = PM_ERR_LOGREC;
@@ -501,21 +506,21 @@ __pmLogPutDesc(__pmLogCtl *lcp, const pmDesc *dp, int numnames, char **names)
 	if (fwrite(&h, 1, sizeof(__pmLogHdr), f) != sizeof(__pmLogHdr) ||
 	    fwrite(odp, 1, sizeof(pmDesc), f) != sizeof(pmDesc) ||
             fwrite(&onumnames, 1, sizeof(numnames), f) != sizeof(numnames))
-		return -errno;
+		return -oserror();
 
         /* write out the names */
         for (i = 0; i < numnames; i++) {
 	    int slen = (int)strlen(names[i]);
 	    olen = htonl(slen);
             if (fwrite(&olen, 1, LENSIZE, f) != LENSIZE)
-                return -errno;
+                return -oserror();
             if (fwrite(names[i], 1, slen, f) != slen)
-                return -errno;
+                return -oserror();
         }
 
 	olen = htonl(len);
 	if (fwrite(&olen, 1, LENSIZE, f) != LENSIZE)
-	    return -errno;
+	    return -oserror();
     }
     else {
 	h.len = htonl(len);
@@ -523,7 +528,7 @@ __pmLogPutDesc(__pmLogCtl *lcp, const pmDesc *dp, int numnames, char **names)
 	if (fwrite(&h, 1, sizeof(__pmLogHdr), f) != sizeof(__pmLogHdr) ||
 	    fwrite(odp, 1, sizeof(pmDesc), f) != sizeof(pmDesc) ||
 	    fwrite(&olen, 1, LENSIZE, f) != LENSIZE)
-		return -errno;
+		return -oserror();
     }
 
     /*
@@ -531,7 +536,7 @@ __pmLogPutDesc(__pmLogCtl *lcp, const pmDesc *dp, int numnames, char **names)
      * may re-use *dp
      */
     if ((tdp = (pmDesc *)malloc(sizeof(pmDesc))) == NULL)
-	return -errno;
+	return -oserror();
     *tdp = *dp;		/* struct assignment */
     return __pmHashAdd((int)dp->pmid, (void *)tdp, &lcp->l_hashpmid);
 }
@@ -560,9 +565,11 @@ searchindom(__pmLogCtl *lcp, pmInDom indom, __pmTimeval *tp)
 	    if (__pmTimevalSub(&idp->stamp, tp) <= 0)
 		break;
 #ifdef PCP_DEBUG
-	    if (pmDebug & DBG_TRACE_LOGMETA)
+	    if (pmDebug & DBG_TRACE_LOGMETA) {
+		fprintf(stderr, "request @ %s is ", StrTimeval(tp));
 		fprintf(stderr, "too early for indom @ %s\n",
 		    StrTimeval(&idp->stamp));
+	    }
 #endif
 	}
 	if (idp == NULL)
@@ -675,7 +682,7 @@ __pmLogPutInDom(__pmLogCtl *lcp, pmInDom indom, const __pmTimeval *tp,
 
     real_numinst = numinst > 0 ? numinst : 0;
     if ((stridx = (int *)malloc(real_numinst * sizeof(stridx[0]))) == NULL)
-	return -errno;
+	return -oserror();
 
     h.len = (int)sizeof(__pmLogHdr) + (int)sizeof(*tp) + (int)sizeof(indom) +
 	    (int)sizeof(numinst) +
@@ -712,11 +719,11 @@ __pmLogPutInDom(__pmLogCtl *lcp, pmInDom indom, const __pmTimeval *tp,
     wlen += fwrite(&h.len, 1, sizeof(h.len), lcp->l_mdfp);
     free(stridx);
 
-    if (wlen != ntohl(h.len)) {
+    if (wlen != (int)ntohl(h.len)) {
 	pmprintf("__pmLogPutInDom: wrote %d, expected %d: %s\n",
-	    wlen, ntohl(h.len), strerror(errno));
+	    wlen, (int)ntohl(h.len), osstrerror());
 	pmflush();
-	return -errno;
+	return -oserror();
     }
 
     sts = addindom(lcp, indom, tp, numinst, instlist, namelist, NULL, 0);
@@ -797,7 +804,7 @@ pmNameInDomArchive(pmInDom indom, int inst, char **name)
 	    for (j = 0; j < idp->numinst; j++) {
 		if (idp->instlist[j] == inst) {
 		    if ((*name = strdup(idp->namelist[j])) == NULL)
-			n = -errno;
+			n = -oserror();
 		    else
 			n = 0;
 		    return n;
