@@ -10,12 +10,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
+#ifdef ASYNC_API
 #include "pmapi.h"
 #include "impl.h"
 #include <assert.h>
@@ -128,11 +125,6 @@ tv_sub(const struct timeval *a, const struct timeval *b)
     return (t.tv_sec * 1000 + t.tv_usec / 1000);
 }
 
-static void
-tv_now(struct timeval *tv)
-{
-    gettimeofday(tv, NULL);
-}
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
 
 int
@@ -301,9 +293,8 @@ pmLoopRegisterSignal(
     if (sig < 0 || sig >= SIGMAX)
 	return -EINVAL;
 
-    if ((ss = (loop_signal_t *)malloc(sizeof(loop_signal_t))) != NULL) {
-	return (-ENOMEM);
-    }
+    if ((ss = (loop_signal_t *)malloc(sizeof(loop_signal_t))) != NULL)
+	return -ENOMEM;
 
     ss->tag = next_tag++;
     ss->callback = callback;
@@ -313,19 +304,18 @@ pmLoopRegisterSignal(
     ss->next = signals[sig];
     signals[sig] = ss;
 
-    if (doinstall)
-    {
+    if (doinstall) {
 	struct sigaction sa;
 
 	memset(&sa, 0, sizeof(sa));
 	sa.sa_flags = 0;
 	sa.sa_handler = loop_sig_handler;
 	if (sigaction(sig, &sa, NULL) < 0) {
-	    int ee = errno;
+	    int ee = oserror();
 
 	    __pmNotifyErr(LOG_WARNING, 
-			  "sigaction failed - %s", strerror(ee));
-	    return (-ee);
+			  "sigaction failed - %s", osstrerror());
+	    return -ee;
 	}
     }
 
@@ -365,7 +355,7 @@ pmLoopUnregisterSignal(int tag)
 	    sa.sa_handler = SIG_DFL;
 	    if (sigaction(sig, &sa, NULL) < 0) {
 		__pmNotifyErr(LOG_WARNING, 
-			      "sigaction failed - %s", strerror(errno));
+			      "sigaction failed - %s", osstrerror());
 		return;
 	    }
 	}
@@ -559,7 +549,7 @@ pmLoopUnregisterTimeout(int tag)
 static int
 loop_setup_timeouts(void)
 {
-    tv_now(&poll_start);
+    __pmtimevalNow(&poll_start);
 
     if (idle_list != NULL)
 	return 0;   /* poll() returns immediately */
@@ -606,7 +596,7 @@ loop_adjust_timeout(void)
     if (timeout_list == NULL)
 	return;
 
-    tv_now(&now);
+    __pmtimevalNow(&now);
     spent = tv_sub(&now, &poll_start);
     if (spent >= timeout_list->delay) {
 	timeout_list->delay = 0;
@@ -638,10 +628,10 @@ pmLoopRegisterChild(
     int doinstall;
 
     if (pmLoopDebug)
-	__pmNotifyErr(LOG_DEBUG,"loop_register_child: pid=%d callback=%p closure=%p",
-		(int)pid, callback, closure);
+	__pmNotifyErr(LOG_DEBUG,"loop_register_child: pid=%" FMT_PID " callback=%p closure=%p",
+		pid, callback, closure);
 
-    if (pid <= 0)
+    if (pid <= (pid_t)0)
 	return -1;
     if ((cc = (loop_child_t *)malloc(sizeof(loop_child_t))) == NULL) {
 	return (-ENOMEM);
@@ -724,7 +714,7 @@ loop_dispatch_children(void)
 	{
 	    nextcc = cc->next;
 
-	    if (r == cc->pid) {
+	    if (r == (int)cc->pid) {
 		loop_main_t *lm = main_stack;
 		int isdone;
 
@@ -837,44 +827,34 @@ pmLoopMain(void)
     main_stack = &lmain;
 
     lmain.running = 1;
-    while (lmain.running)
-    {
+    while (lmain.running) {
 	int ee;
 
-	if ((ee = loop_setup_inputs()) < 0) { 
-		return (ee);
-	}
+	if ((ee = loop_setup_inputs()) < 0)
+	    return ee;
 	timeout = loop_setup_timeouts();
 	loop_dispatch_idle();
 
-	if ((ee == 0) && (timeout == -1) && (idle_list == NULL)) {
-	    return (0);
-	}
+	if ((ee == 0) && (timeout == -1) && (idle_list == NULL))
+	    return 0;
 
 	r = poll(pfd, num_inputs, timeout);
-
-	if (r < 0)
-	{
-	    if (errno == EINTR)
-	    {
+	if (r < 0) {
+	    if (oserror() == EINTR) {
 		loop_dispatch_signals();
 		if (child_pending)
 		    loop_dispatch_children();
 	    	continue;
 	    }
-	    __pmNotifyErr(LOG_ERR,
-			  "pmLoopMain: poll failed - %s", strerror(errno));
+	    __pmNotifyErr(LOG_ERR, "pmLoopMain: poll failed - %s",
+			  osstrerror());
 	    break;
-	}
-	else if (r == 0)
-	{
+	} else if (r == 0) {
 	    if (timeout > 0)
 		loop_dispatch_timeouts();
 	    else
 		loop_adjust_timeout();
-	}
-	else
-	{
+	} else {
 	    loop_dispatch_inputs();
 	    loop_adjust_timeout();
 	}
@@ -884,7 +864,8 @@ pmLoopMain(void)
     assert(lmain.current_child == NULL);
     assert(lmain.current_timeout == NULL);
     main_stack = lmain.next;
-    return (0);
+    return 0;
 }
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
+#endif /*ASYNC_API*/
