@@ -10,10 +10,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include "pmapi.h"
@@ -38,16 +34,15 @@ typedef struct {
 } x_error_t;
 
 int
-__pmSendError(int fd, int mode, int code)
+__pmSendError(int fd, int from, int code)
 {
     p_error_t	*pp;
 
-    if (mode == PDU_ASCII)
-	return PM_ERR_NOASCII;
     if ((pp = (p_error_t *)__pmFindPDUBuf(sizeof(p_error_t))) == NULL)
-	return -errno;
+	return -oserror();
     pp->hdr.len = sizeof(p_error_t);
     pp->hdr.type = PDU_ERROR;
+    pp->hdr.from = from;
 
     if (__pmVersionIPC(fd) == PDU_VERSION1)
 	pp->code = XLATE_ERR_2TO1(code);
@@ -67,16 +62,15 @@ __pmSendError(int fd, int mode, int code)
 }
 
 int
-__pmSendXtendError(int fd, int mode, int code, int datum)
+__pmSendXtendError(int fd, int from, int code, int datum)
 {
     x_error_t	*pp;
 
-    if (mode == PDU_ASCII)
-	return PM_ERR_NOASCII;
     if ((pp = (x_error_t *)__pmFindPDUBuf(sizeof(x_error_t))) == NULL)
-	return -errno;
+	return -oserror();
     pp->hdr.len = sizeof(x_error_t);
     pp->hdr.type = PDU_ERROR;
+    pp->hdr.from = from;
 
     pp->code = htonl(XLATE_ERR_2TO1(code));
     pp->datum = datum; /* NOTE: caller must swab this */
@@ -85,12 +79,10 @@ __pmSendXtendError(int fd, int mode, int code, int datum)
 }
 
 int
-__pmDecodeError(__pmPDU *pdubuf, int mode, int *code)
+__pmDecodeError(__pmPDU *pdubuf, int *code)
 {
     p_error_t	*pp;
 
-    if (mode == PDU_ASCII)
-	return PM_ERR_NOASCII;
     pp = (p_error_t *)pdubuf;
     if (__pmLastVersionIPC() == PDU_VERSION1)
 	*code = XLATE_ERR_1TO2((int)ntohl(pp->code));
@@ -106,13 +98,11 @@ __pmDecodeError(__pmPDU *pdubuf, int mode, int *code)
 }
 
 int
-__pmDecodeXtendError(__pmPDU *pdubuf, int mode, int *code, int *datum)
+__pmDecodeXtendError(__pmPDU *pdubuf, int *code, int *datum)
 {
     x_error_t	*pp = (x_error_t *)pdubuf;
     int		sts;
 
-    if (mode == PDU_ASCII)
-	return PM_ERR_NOASCII;
     /*
      * it is ALWAYS a PCP 1.x error code here
      */
