@@ -10,10 +10,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include <limits.h>
@@ -600,7 +596,7 @@ pmParseTimeWindow(
     struct timeval  aoffset;
     struct timeval  tval;
     const char	    *scan;
-    __int64_t	    delta;
+    __int64_t	    delta = 0;	/* initialize to pander to gcc */
     __int64_t 	    align;
     __int64_t 	    blign;
     int		    sts = 1;
