@@ -11,10 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 /*
@@ -24,7 +20,9 @@
 #include <ctype.h>
 #include "pmapi.h"
 #include "impl.h"
-
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 
 static void *
 parseAlloc(const char *func, size_t need)
@@ -82,11 +80,11 @@ pmParseMetricSpec(
     const char	    *h_start = NULL;	/* host name */
     const char	    *h_end = NULL;
     const char	    *a_start = NULL;	/* archive name */
-    const char	    *a_end;
+    const char	    *a_end = NULL;
     const char	    *m_start = NULL;	/* metric name */
-    const char	    *m_end;
+    const char	    *m_end = NULL;
     const char	    *i_start = NULL;	/* instance names */
-    const char	    *i_end;
+    const char	    *i_end = NULL;
     char	    *i_str = NULL;	/* temporary instance names */
     char	    *i_scan;
     int		    ninst;		/* number of instance names */
@@ -94,7 +92,6 @@ pmParseMetricSpec(
     const char	    *pull;
     int		    length;
     int		    i;
-    int             type = 0;
     int		    inquote = 0;	/* true if within quotes */
 
     scan = spec;
@@ -114,7 +111,6 @@ pmParseMetricSpec(
     if (mark == NULL) mark = &scan[strlen(scan)-1];
     while (mark >= scan) {
 	if (*mark == ':') {
-	    type = PM_CONTEXT_HOST;
 	    h_start = scan;
 	    h_end = mark-1;
 	    while (h_end >= scan && isspace((int)*h_end)) h_end--;
@@ -127,7 +123,6 @@ pmParseMetricSpec(
 	    break;
 	}
 	else if (*mark == '/') {
-	    type = PM_CONTEXT_ARCHIVE;
 	    a_start = scan;
 	    a_end = mark-1;
 	    while (a_end >= scan && isspace((int)*a_end)) a_end--;
@@ -436,8 +431,11 @@ __pmParseHostSpec(
 	    if (*s == ':') {
 		for (++s, start = s; s != NULL; s++) {
 		    if (*s == ',' || *s == '@' || *s == '\0') {
-			if (s == start)
-			    continue;
+			if (s - start < 1) {
+			    hostError(spec, s, "missing port", errmsg);
+			    sts = PM_ERR_GENERIC;
+			    goto fail;
+			}
 			int port = atoi(start);
 			sts = __pmAddHostPorts(&hsp[nhosts-1], &port, 1);
 			if (sts < 0)
