@@ -10,17 +10,13 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include "pmapi.h"
 #include "impl.h"
 
 int
-__pmStuffValue(const pmAtomValue *avp, int aggr_len, pmValue *vp, int type)
+__pmStuffValue(const pmAtomValue *avp, pmValue *vp, int type)
 {
     void	*src;
     size_t	need, body;
@@ -44,8 +40,15 @@ __pmStuffValue(const pmAtomValue *avp, int aggr_len, pmValue *vp, int type)
 	    break;
 
 	case PM_TYPE_AGGREGATE:
-	    body = aggr_len;
-	    src  = avp->vp;
+	    /*
+	     * vbp field of pmAtomValue points to a dynamically allocated
+	     * pmValueBlock ... the vlen and vtype fields MUST have been
+	     * already set up.
+	     * A new pmValueBlock header will be allocated below, so adjust
+	     * the length here (PM_VAL_HDR_SIZE will be added back later).
+	     */
+	    body = avp->vbp->vlen - PM_VAL_HDR_SIZE;
+	    src  = avp->vbp->vbuf;
 	    break;
 	    
 	case PM_TYPE_STRING:
@@ -54,11 +57,19 @@ __pmStuffValue(const pmAtomValue *avp, int aggr_len, pmValue *vp, int type)
 	    break;
 
 	case PM_TYPE_AGGREGATE_STATIC:
-	    vp->value.pval = (pmValueBlock *)avp->vp;
+	case PM_TYPE_EVENT:
+	    /*
+	     * vbp field of pmAtomValue points to a statically allocated
+	     * pmValueBlock ... the vlen and vtype fields MUST have been
+	     * already set up and are not modified here
+	     *
+	     * DO NOT make a copy of the value in this case
+	     */
+	    vp->value.pval = avp->vbp;
 	    return PM_VAL_SPTR;
 
 	default:
-	    return PM_ERR_GENERIC;
+	    return PM_ERR_TYPE;
     }
     need = body + PM_VAL_HDR_SIZE;
     if (body == sizeof(__int64_t)) {
@@ -68,7 +79,7 @@ __pmStuffValue(const pmAtomValue *avp, int aggr_len, pmValue *vp, int type)
 		(need < sizeof(pmValueBlock)) ? sizeof(pmValueBlock) : need);
     }
     if (vp->value.pval == NULL)
-	return -errno;
+	return -oserror();
     vp->value.pval->vlen = (int)need;
     vp->value.pval->vtype = type;
     memcpy((void *)vp->value.pval->vbuf, (void *)src, body);
