@@ -133,7 +133,7 @@ pmTimeStateSetup(
 	pmtime->end = last;
     } else {
 	pmtime->source = PM_SOURCE_HOST;
-	gettimeofday(&pmtime->position, NULL);
+	__pmtimevalNow(&pmtime->position);
     }
     if (tz == NULL) {
 	tz = __pmTimezone();
@@ -151,7 +151,7 @@ pmTimeStateSetup(
     pmtime->length += tzlen + strlen(tz_label) + 1;
     pmtime = realloc(pmtime, pmtime->length);
     if (!pmtime) {
-	fprintf(stderr, "%s: realloc: %s\n", pmProgname, strerror(errno));
+	fprintf(stderr, "%s: realloc: %s\n", pmProgname, osstrerror());
 	exit(EXIT_FAILURE);
     }
     strcpy(pmtime->data, tz);
