@@ -10,10 +10,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include "pmapi.h"
@@ -35,12 +31,15 @@ typedef struct entry {
     int			inst;
     char		*name;
     int			hashlen;	/* smaller of strlen(name) and chars to first space */
+    int			keylen;		/* > 0 if have key from pmdaCacheStoreKey() */
+    void		*key;		/* != NULL if have key from pmdaCacheStoreKey() */
     int			state;
     void		*private;
     time_t		stamp;
 } entry_t;
 
 #define VERSION 1	/* version of external file format */
+#define MAX_HASH_TRY	10
 
 /*
  * linked list of cache headers
@@ -58,6 +57,7 @@ typedef struct hdr {
     int			nentry;		/* number of entries */
     int			ins_mode;	/* see insert_cache() */
     int			hstate;		/* dirty/clean state for save_cache() */
+    int			keyhash_cnt[MAX_HASH_TRY];
 } hdr_t;
 
 /* bitfields for hstate */
@@ -89,6 +89,20 @@ hash_str(const char *str, int len)
     return hash(str, len, 0);
 }
 
+static void
+KeyStr(FILE *f, int keylen, const char *key)
+{
+    int		i;
+    if (keylen > 0) {
+	fprintf(f, "[key=0x");
+	for (i = 0; i < keylen; i++, key++)
+	    fprintf(f, "%02x", (*key & 0xff));
+	fputc(']', f);
+    }
+    else
+	fprintf(f, "[no key]");
+}
+
 /*
  * The magic "match up to a space" instance name matching ...
  *
@@ -118,10 +132,33 @@ name_eq(entry_t *e, const char *name, int hashlen)
     return -1;
 }
 
+static int
+key_eq(entry_t *e, int keylen, const char *key)
+{
+    const char	*ekp;
+    const char	*kp;
+    int		i;
+
+    if (e->keylen != keylen)
+	return 0;
+
+    ekp = (const char *)e->key;
+    kp = (const char *)key;
+    for (i = 0; i < keylen; i++) {
+	if (*ekp != *kp)
+	    return 0;
+	ekp++;
+	kp++;
+    }
+
+    return 1;
+}
+
 static hdr_t *
 find_cache(pmInDom indom, int *sts)
 {
     hdr_t	*h;
+    int		i;
 
     for (h = base; h != NULL; h = h->next) {
 	if (h->indom == indom)
@@ -147,6 +184,8 @@ find_cache(pmInDom indom, int *sts)
     h->nentry = 0;
     h->ins_mode = 0;
     h->hstate = 0;
+    for (i = 0; i < MAX_HASH_TRY; i++)
+	h->keyhash_cnt[i] = 0;
     return h;
 }
 
@@ -203,6 +242,7 @@ static void
 dump(FILE *fp, hdr_t *h, int do_hash)
 {
     entry_t	*e;
+    int		i;
 
     fprintf(fp, "pmdaCacheDump: indom %s: nentry=%d ins_mode=%d hstate=%d hsize=%d\n",
 	pmInDomStr(h->indom), h->nentry, h->ins_mode, h->hstate, h->hsize);
@@ -216,12 +256,34 @@ dump(FILE *fp, hdr_t *h, int do_hash)
 		e->private, e->name);
 	    if (strlen(e->name) > e->hashlen)
 		fprintf(fp, " [match len=%d]", e->hashlen);
+	    if (e->keylen > 0) {
+		fputc(' ', fp);
+		KeyStr(fp, e->keylen, (const char *)e->key);
+	    }
 	    fputc('\n', fp);
 	}
     }
+
     if (do_hash == 0)
 	return;
 
+    for (i = 0; i < MAX_HASH_TRY; i++) {
+	if (h->keyhash_cnt[i])
+	    break;
+    }
+
+    if (i < MAX_HASH_TRY) {
+	fprintf(fp, "pmdaCacheStoreKey hash stats ...\n");
+	for (i = 0; i < MAX_HASH_TRY; i++) {
+	    if (h->keyhash_cnt[i] != 0) {
+		if (i == 0)
+		    fprintf(fp, "hash once: %d times\n", h->keyhash_cnt[i]);
+		else
+		    fprintf(fp, "%d hash attempts: %d times\n", i+1, h->keyhash_cnt[i]);
+	    }
+	}
+    }
+
     if (h->ctl_inst != NULL) {
 	int	i;
 	fprintf(fp, "inst hash\n");
@@ -260,7 +322,7 @@ find_inst(hdr_t *h, int inst)
     entry_t	*e;
 
     for (e = h->first; e != NULL; e = e->next) {
-	if (e->state != PMDA_CACHE_EMPTY && e->inst == inst)
+	if (e->inst == inst && e->state != PMDA_CACHE_EMPTY)
 	    break;
     }
     return e;
@@ -285,7 +347,7 @@ find_entry(hdr_t *h, const char *name, int inst, int *sts)
 	    /* no hash, use linear search */
 	    return find_inst(h, inst);
 	for (e = h->ctl_inst[inst & h->hbits]; e != NULL; e = e->h_inst) {
-	    if (e->state != PMDA_CACHE_EMPTY && e->inst == inst)
+	    if (e->inst == inst && e->state != PMDA_CACHE_EMPTY)
 		return e;
 	}
     }
@@ -478,7 +540,8 @@ reorder:
  * unused inst value.
  *
  * If inst is _not_ PM_IN_NULL, we're being called from load_cache
- * and there is no choice, but to walk the list.
+ * or pmdaCacheStoreKey() and the inst is known ... so we need to
+ * check for possible duplicate entries.
  */
 static entry_t *
 insert_cache(hdr_t *h, const char *name, int inst, int *sts)
@@ -492,26 +555,47 @@ insert_cache(hdr_t *h, const char *name, int inst, int *sts)
     *sts = 0;
 
     if (inst != PM_IN_NULL) {
-	/* load_cache case, inst is known */
-	for (e = h->first; e != NULL; e = e->next) {
-	    /*
-	     * check if instance id or instance name already in cache
-	     * ... if id and name are the the same, keep the existing
-	     * one and ignore the one from load_cache (in particular
-	     * state is not reset to inactive), otherwise do nothing
-	     * and return an error indication
-	     */
-	    if (e->inst == inst) {
-		if (name_eq(e, name, hashlen) != 1)
-		    *sts = PM_ERR_INST;
-		return e;
-	    }
-	    if (name_eq(e, name, hashlen) == 1) {
+	/*
+	 * Check if instance id or instance name already in cache
+	 * ... if id and name are the the same, keep the existing one
+	 * and ignore the new one (in particular state is not reset to
+	 * inactive).
+	 * If one matches but the other is different, keep the
+	 * matching entry, but return an error as a warning.
+	 * If both fail to match, we're OK to insert the new entry,
+	 * although we need to run down the list quickly to find the
+	 * correct place to insert the new one.
+	 */
+	e = find_entry(h, NULL, inst, sts);
+	if (e != NULL) {
+	    if (name_eq(e, name, hashlen) != 1) {
+		/* instance id the same, different name */
+#ifdef PCP_DEBUG
+		if (pmDebug & DBG_TRACE_INDOM) {
+		    fprintf(stderr, "pmdaCache: store: indom %s: instance %d ", pmInDomStr(h->indom), e->inst);
+		    fprintf(stderr, " in cache, name \"%s\" does not match new entry \"%s\"\n", e->name, name);
+		}
+#endif
 		*sts = PM_ERR_INST;
-		return e;
 	    }
+	    return e;
+	}
+	e = find_entry(h, name, PM_IN_NULL, sts);
+	if (e != NULL) {
+#ifdef PCP_DEBUG
+	    if (pmDebug & DBG_TRACE_INDOM) {
+		fprintf(stderr, "pmdaCacheStoreKey: indom %s: instance \"%s\"", pmInDomStr(h->indom), e->name);
+		fprintf(stderr, " in cache, id %d does not match new entry %d\n", e->inst, inst);
+	    }
+#endif
+	    *sts = PM_ERR_INST;
+	    return e;
+	}
+	for (e = h->first; e != NULL; e = e->next) {
 	    if (e->inst < inst)
 		last_e = e;
+	    else if (e->inst > inst)
+		break;
 	}
     }
 
@@ -584,6 +668,7 @@ retry:
     e->inst = inst;
     e->name = dup;
     e->hashlen = get_hashlen(dup);
+    e->key = NULL;
     e->state = PMDA_CACHE_INACTIVE;
     e->private = NULL;
     e->stamp = 0;
@@ -623,6 +708,8 @@ load_cache(hdr_t *h)
     int		cnt;
     int		x;
     int		inst;
+    int		keylen;
+    void	*key;
     int		s;
     char	buf[1024];	/* input line buffer, is this big enough? */
     char	*p;
@@ -639,7 +726,7 @@ load_cache(hdr_t *h)
     snprintf(filename, sizeof(filename), "%s%cconfig%cpmda%c%s",
 		vdp, sep, sep, sep, pmInDomStr(h->indom));
     if ((fp = fopen(filename, "r")) == NULL)
-	return -errno;
+	return -oserror();
     if (fgets(buf, sizeof(buf), fp) == NULL) {
 	__pmNotifyErr(LOG_ERR, 
 	     "pmdaCacheOp: %s: empty file?", filename);
@@ -677,6 +764,41 @@ load_cache(hdr_t *h)
 	}
 	while (*p && isascii((int)*p) && isspace((int)*p))
 	    p++;
+	if (*p == '[') {
+	    char	*pend;
+	    char	*q;
+	    int		i;
+	    int		tmp;
+	    p++;
+	    pend = p;
+	    while (*pend && *pend != ']')
+		pend++;
+	    if (*pend != ']')
+		goto bad;
+	    /*
+	     * convert key in place ...
+	     */
+	    keylen = (pend - p) / 2;
+	    if ((key = malloc(keylen)) == NULL) {
+		__pmNotifyErr(LOG_ERR, 
+		     "load_cache: indom %s: unable to allocate memory for keylen=%d",
+		     pmInDomStr(h->indom), keylen);
+		return PM_ERR_GENERIC;
+	    }
+	    q = key;
+	    for (i = 0; i < keylen; i++) {
+		sscanf(p, "%2x", &tmp);
+		*q++ = (tmp & 0xff);
+		p += 2;
+	    }
+	    p += 2;
+	    while (*p && isascii((int)*p) && isspace((int)*p))
+		p++;
+	}
+	else {
+	    keylen = 0;
+	    key = NULL;
+	}
 	if (*p == '\0') {
 bad:
 	    __pmNotifyErr(LOG_ERR, 
@@ -692,6 +814,8 @@ bad:
 		"pmdaCacheOp: %s: loading instance %d (\"%s\") ignored, already in cache as %d (\"%s\")",
 		filename, inst, p, e->inst, e->name);
 	}
+	e->keylen = keylen;
+	e->key = key;
 	e->stamp = x;
     }
     fclose(fp);
@@ -730,7 +854,7 @@ save_cache(hdr_t *h, int hstate)
     snprintf(filename, sizeof(filename), "%s%cconfig%cpmda%c%s",
 		vdp, sep, sep, sep, pmInDomStr(h->indom));
     if ((fp = fopen(filename, "w")) == NULL)
-	return -errno;
+	return -oserror();
     fprintf(fp, "%d %d\n", VERSION, h->ins_mode);
 
     now = time(NULL);
@@ -740,7 +864,16 @@ save_cache(hdr_t *h, int hstate)
 	    continue;
 	if (e->stamp == 0)
 	    e->stamp = now;
-	fprintf(fp, "%d %d %s\n", e->inst, (int)e->stamp, e->name);
+	fprintf(fp, "%d %d", e->inst, (int)e->stamp);
+	if (e->keylen > 0) {
+	    char	*p = (char *)e->key;
+	    int		i;
+	    fprintf(fp, " [");
+	    for (i = 0; i < e->keylen; i++, p++)
+		fprintf(fp, "%02x", (*p & 0xff));
+	    fputc(']', fp);
+	}
+	fprintf(fp, " %s\n", e->name);
 	cnt++;
     }
     fclose(fp);
@@ -781,8 +914,8 @@ __pmdaCacheDump(FILE *fp, pmInDom indom, int do_hash)
     }
 }
 
-int
-pmdaCacheStore(pmInDom indom, int flags, const char *name, void *private)
+static int
+store(pmInDom indom, int flags, const char *name, pmInDom inst, int keylen, const char *key, void *private)
 {
     hdr_t	*h;
     entry_t	*e;
@@ -791,12 +924,21 @@ pmdaCacheStore(pmInDom indom, int flags, const char *name, void *private)
     if ((h = find_cache(indom, &sts)) == NULL)
 	return sts;
 
-    if ((e = find_entry(h, name, PM_IN_NULL, &sts)) == NULL) {
+    if ((e = find_entry(h, name, inst, &sts)) == NULL) {
 
-	if (flags != PMDA_CACHE_ADD)
+	if (flags != PMDA_CACHE_ADD) {
+#ifdef PCP_DEBUG
+	    if (pmDebug & DBG_TRACE_INDOM) {
+		fprintf(stderr, "pmdaCache store: indom %s: instance \"%s\"", pmInDomStr(indom), name);
+		if (inst != PM_IN_NULL)
+		    fprintf(stderr, " (%d)", inst);
+		fprintf(stderr, " not in cache: flags=%d not allowed\n", flags);
+	    }
+#endif
 	    return PM_ERR_INST;
+	}
 
-	if ((e = insert_cache(h, name, PM_IN_NULL, &sts)) == NULL)
+	if ((e = insert_cache(h, name, inst, &sts)) == NULL)
 	    return sts;
 	h->hstate |= DIRTY_INSTANCE;	/* added a new entry */
     }
@@ -813,6 +955,18 @@ pmdaCacheStore(pmInDom indom, int flags, const char *name, void *private)
 
     switch (flags) {
 	case PMDA_CACHE_ADD:
+	    e->keylen = keylen;
+	    if (keylen > 0) {
+		if ((e->key = malloc(keylen)) == NULL) {
+		    __pmNotifyErr(LOG_ERR, 
+			 "store: indom %s: unable to allocate memory for keylen=%d",
+			 pmInDomStr(indom), keylen);
+		    return PM_ERR_GENERIC;
+		}
+		memcpy(e->key, key, keylen);
+	    }
+	    else
+		e->key = NULL;
 	    e->state = PMDA_CACHE_ACTIVE;
 	    e->private = private;
 	    e->stamp = 0;		/* flag, updated at next cache_save() */
@@ -840,12 +994,155 @@ pmdaCacheStore(pmInDom indom, int flags, const char *name, void *private)
     return e->inst;
 }
 
+int
+pmdaCacheStore(pmInDom indom, int flags, const char *name, void *private)
+{
+    if (indom == PM_INDOM_NULL)
+	return PM_ERR_INDOM;
+
+    return store(indom, flags, name, PM_IN_NULL, 0, NULL, private);
+}
+
+/*
+ * Generate a new 31-bit (positive) instance number from a key provided
+ * as a ``hint'' via key[] (first keylen bytes) or name[] if keylen < 1
+ * or key == NULL ... useful for compressing natural 64-bit or larger
+ * instance identifiers into the 31-bits required for the PCP APIs
+ * and PDUs.
+ */
+int
+pmdaCacheStoreKey(pmInDom indom, int flags, const char *name, int keylen, const void *key, void *private)
+{
+    int		inst;
+    int		sts;
+    int		i;
+    __uint32_t	try = 0;
+    hdr_t	*h;
+    entry_t	*e;
+    const char	*mykey;
+    int		mykeylen;
+
+    if (indom == PM_INDOM_NULL)
+	return PM_ERR_INDOM;
+
+    if (flags != PMDA_CACHE_ADD)
+	return store(indom, flags, name, PM_IN_NULL, 0, NULL, private);
+
+    /*
+     * This is the PMDA_CACHE_ADD case, so need to find an instance id
+     */
+    if ((h = find_cache(indom, &sts)) == NULL)
+	return sts;
+
+    if (keylen < 1 || key == NULL) {
+	/* use name[] instead of keybuf[] */
+	mykey = (const char *)name;
+	mykeylen = strlen(name);
+    }
+    else {
+	mykey = key;
+	mykeylen = keylen;
+    }
+
+    if ((e = find_entry(h, name, PM_IN_NULL, &sts)) != NULL) {
+	/*
+	 * cache entry already exists for this name ...
+	 * if keys are not equal => failure
+	 */
+	if (key_eq(e, mykeylen, mykey) == 0) {
+#ifdef PCP_DEBUG
+	    if (pmDebug & DBG_TRACE_INDOM) {
+		fprintf(stderr, "pmdaCacheStoreKey: indom %s: instance \"%s\" (%d) in cache ", pmInDomStr(indom), e->name, e->inst);
+		KeyStr(stderr, e->keylen, (const char *)e->key);
+		fprintf(stderr, " does not match new entry ");
+		KeyStr(stderr, mykeylen, mykey);
+		fputc('\n', stderr);
+	    }
+#endif
+	    return PM_ERR_INST;
+	}
+	/* keys the same, use inst from existing entry */
+	inst = e->inst;
+    }
+    else {
+	/* we're in the inst guessing game ... */
+	for (i = 0; i < MAX_HASH_TRY; i++) {
+	    try = hash(mykey, mykeylen, try);
+	    /* strip top bit ... instance id must be positive */
+	    inst = try & ~(1 << (8*sizeof(__uint32_t)-1));
+	    e = find_entry(h, NULL, inst, &sts);
+	    if (e == NULL) {
+		h->keyhash_cnt[i]++;
+		break;
+	    }
+	    /*
+	     * Found matching entry using the guessed inst ...
+	     *
+	     * If the key[]s are the same and the name[]s are the same
+	     * then the matching entry is already in the cache, so use
+	     * this instance identifier.
+	     *
+	     * If the key[]s match, but the name[]s are different, this
+	     * is an error (duplicate instance name).
+	     *
+	     * If the names[] match, but the key[]s are different, this
+	     * is an error (duplicate key).
+	     *
+	     * Otherwise instance id is in use for another instance, so
+	     * keep trying by rehashing.
+	     */
+	    if (strcmp(e->name, name) == 0) {
+		if (key_eq(e, mykeylen, mykey) == 1)
+		    break;
+#ifdef PCP_DEBUG
+		if (pmDebug & DBG_TRACE_INDOM) {
+		    fprintf(stderr, "pmdaCacheStoreKey: indom %s: instance \"%s\" (%d) in cache, ", pmInDomStr(indom), e->name, e->inst);
+		    KeyStr(stderr, e->keylen, (const char *)e->key);
+		    fprintf(stderr, " does not match new entry ");
+		    KeyStr(stderr, mykeylen, mykey);
+		    fputc('\n', stderr);
+		}
+#endif
+		return PM_ERR_INST;
+	    }
+	    else if (key_eq(e, mykeylen, mykey) == 1) {
+#ifdef PCP_DEBUG
+		if (pmDebug & DBG_TRACE_INDOM) {
+		    fprintf(stderr, "pmdaCacheStoreKey: indom %s: instance %d ", pmInDomStr(indom), e->inst);
+		    KeyStr(stderr, e->keylen, (const char *)e->key);
+		    fprintf(stderr, " in cache, name \"%s\" does not match new entry \"%s\"\n", e->name, name);
+		}
+#endif
+		return PM_ERR_INST;
+	    }
+	}
+	if (i == MAX_HASH_TRY) {
+	    /* failed after MAX_HASH_TRY rehash attempts ... */
+	    __pmNotifyErr(LOG_ERR, 
+		 "pmdaCacheStoreKey: indom %s: unable allocate a new id for instance \"%s\" based on a key of %d bytes\n",
+		 pmInDomStr(h->indom), name, keylen);
+	    return PM_ERR_GENERIC;
+	}
+
+	/*
+	 * when using key[] or name[] as a hint, we permanently change
+	 * to PMDA_CACHE_REUSE mode
+	 */
+	h->ins_mode = 1;
+    }
+
+    return store(indom, flags, name, inst, mykeylen, mykey, private);
+}
+
 int pmdaCacheOp(pmInDom indom, int op)
 {
     hdr_t	*h;
     entry_t	*e;
     int		sts;
 
+    if (indom == PM_INDOM_NULL)
+	return PM_ERR_INDOM;
+
     if (op == PMDA_CACHE_CHECK) {
 	/* is there a cache for this one? */
 	for (h = base; h != NULL; h = h->next) {
@@ -941,6 +1238,10 @@ int pmdaCacheOp(pmInDom indom, int op)
 	    return -1;
 
 	case PMDA_CACHE_DUMP:
+	    dump(stderr, h, 0);
+	    return 0;
+
+	case PMDA_CACHE_DUMP_ALL:
 	    dump(stderr, h, 1);
 	    return 0;
 
@@ -955,6 +1256,9 @@ int pmdaCacheLookupName(pmInDom indom, const char *name, int *inst, void **priva
     entry_t	*e;
     int		sts;
 
+    if (indom == PM_INDOM_NULL)
+	return PM_ERR_INDOM;
+
     if ((h = find_cache(indom, &sts)) == NULL)
 	return sts;
 
@@ -978,6 +1282,9 @@ int pmdaCacheLookup(pmInDom indom, int inst, char **name, void **private)
     entry_t	*e;
     int		sts;
 
+    if (indom == PM_INDOM_NULL)
+	return PM_ERR_INDOM;
+
     if ((h = find_cache(indom, &sts)) == NULL)
 	return sts;
 
@@ -994,6 +1301,60 @@ int pmdaCacheLookup(pmInDom indom, int inst, char **name, void **private)
     return e->state;
 }
 
+int pmdaCacheLookupKey(pmInDom indom, const char *name, int keylen, const void *key, char **oname, int *inst, void **private)
+{
+    hdr_t	*h;
+    entry_t	*e;
+    int		sts;
+    const char	*mykey;
+    int		mykeylen;
+
+    if (indom == PM_INDOM_NULL)
+	return PM_ERR_INDOM;
+
+    if ((h = find_cache(indom, &sts)) == NULL)
+	return sts;
+
+    if (keylen < 1 || key == NULL) {
+	/* use name[] instead of keybuf[] */
+	mykey = (const char *)name;
+	mykeylen = strlen(name);
+    }
+    else {
+	mykey = key;
+	mykeylen = keylen;
+    }
+
+    /*
+     * No hash list for key[]s ... have to walk the cache.
+     * pmdaCacheStoreKey() ensures the key[]s are unique, so first match
+     * wins.
+     */
+    walk_cache(h, PMDA_CACHE_WALK_REWIND);
+    while ((e = walk_cache(h, PMDA_CACHE_WALK_NEXT)) != NULL) {
+	if (e->state == PMDA_CACHE_EMPTY)
+	    continue;
+	if (key_eq(e, mykeylen, mykey) == 1) {
+	    if (oname != NULL)
+		*oname = e->name;
+	    if (inst != NULL)
+		*inst = e->inst;
+	    if (private != NULL)
+		*private = e->private;
+	    return e->state;
+	}
+    }
+
+#ifdef PCP_DEBUG
+    if (pmDebug & DBG_TRACE_INDOM) {
+	    fprintf(stderr, "pmdaCacheLookupKey: indom %s: ", pmInDomStr(h->indom));
+	    KeyStr(stderr, mykeylen, mykey);
+	    fprintf(stderr, ": no matching key in cache\n");
+	}
+#endif
+    return PM_ERR_INST;
+}
+
 int pmdaCachePurge(pmInDom indom, time_t recent)
 {
     hdr_t	*h;
@@ -1002,6 +1363,9 @@ int pmdaCachePurge(pmInDom indom, time_t recent)
     int		cnt;
     int		sts;
 
+    if (indom == PM_INDOM_NULL)
+	return PM_ERR_INDOM;
+
     if ((h = find_cache(indom, &sts)) == NULL)
 	return sts;
 
